@@ -66,7 +66,16 @@ def sqrt_real(x):
 # 为了应用自动微分,必须使用torch,截止matlab r2023b还不支持特征分解的自动微分,需要额外的工作量
 # GPU比CPU快150倍卧槽
 # 只支持周期结构
-# 色散的bug暂时还没修......
+
+
+# 我早就想支持材料id优化了，这次正好乘着紫外波带片的契机做一下。
+# 引入（优化层数，材料数目）
+def sigmoid(y, beta, eta=0.5):
+    beta = torch.tensor(beta).to(y.device)
+    z = (torch.tanh(beta * eta) + torch.tanh(beta * (y - eta))) / (
+        torch.tanh(beta * eta) + torch.tanh(beta * (1 - eta))
+    )
+    return z
 
 
 class ETM:
@@ -75,11 +84,12 @@ class ETM:
         period,
         lams,
         material_list,  # 材料列表
+        is_opt_material,
         background_material_id,  # 背景材料序号
         stack_material_id,  # 填充的背景材料id,默认全是
+        surf_material_id,  # (每层材料的初始id)
         height,
-        is_surf,
-        surf_material_id,  # 填充的主材料id,
+        is_opt_surf,
         surf,  # 默认全0
         theta_deg,
         phi_deg,
@@ -95,7 +105,7 @@ class ETM:
         self.device = device
         lams = tensor(lams, device=device).view(-1)
         height = tensor(height, device=device)
-        is_surf = tensor(is_surf, dtype=torch.bool, device=device)
+        is_opt_surf = tensor(is_opt_surf, dtype=torch.bool, device=device)
         surf = tensor(surf, dtype=torch.float32, device=device)
         theta_deg = tensor(theta_deg, device=device)
         phi_deg = tensor(phi_deg, device=device)
@@ -108,10 +118,15 @@ class ETM:
         self.Ty = period[1]
         self.F_u = F_u
         self.F_v = F_v
-        self.stack_material_id = tensor(stack_material_id, device=device)
-        self.surf_material_id = tensor(surf_material_id, device=device)
+        self.stack_material_id = tensor(
+            stack_material_id, device=device, dtype=torch.long
+        )
+        self.is_opt_Material = tensor(is_opt_material, dtype=torch.bool, device=device)
+        self.opt_id_list = torch.nonzero(self.is_opt_Material, as_tuple=False).view(-1)
+        self.num_opt_mat = self.opt_id_list.numel()
+
         self.surf = tensor(surf, device=device, dtype=dtype_real)
-        self.is_surf = is_surf
+        self.is_opt_surf = is_opt_surf
         self.height = tensor(height, device=device)
 
         self.Material_list = []
@@ -120,6 +135,8 @@ class ETM:
             self.Material_list.append(
                 Material(m, self.lams)  # 改回来!![0] * torch.ones_like(self.lams)
             )
+
+        self.surf_logits = self.convert_surf(tensor(surf_material_id, device=device))
         self.n_re = self.Material_list[background_material_id[0]].index
         self.n_tr = self.Material_list[background_material_id[1]].index
         self.er_stack = torch.ones(
@@ -128,14 +145,9 @@ class ETM:
             device=device,
         )
         for z in range(self.er_stack.size(0)):
-            self.er_stack[z] = self.Material_list[self.stack_material_id[z]].index ** 2
-        self.er_surf = torch.ones(
-            [self.surf_material_id.numel(), self.lams.numel()],
-            dtype=dtype,
-            device=device,
-        )
-        for z in range(self.er_surf.size(0)):
-            self.er_surf[z] = self.Material_list[self.surf_material_id[z]].index ** 2
+            self.er_stack[z] = (
+                self.Material_list[int(self.stack_material_id[z])].index ** 2
+            )
 
         self.mat_shape = [surf.size(1), surf.size(2)]  # surf是(z,x,y)
         M = 2 * F_u + 1
@@ -193,7 +205,7 @@ class ETM:
 
         atm = cross(ate, A0, dim=-1)
         norms = torch.norm(atm, dim=-1).view(-1, 1)
-        atm = atm / norm(atm)
+        atm = atm / norms
 
         # 根据TE，TM偏振比例求xy分量
         P = pte * ate + ptm * atm
@@ -226,6 +238,10 @@ class ETM:
         self.Az_re_inv = pinv(self.Az_re, eps)
 
         # 计算复用参数
+        self.allowed_n = torch.stack(
+            [self.Material_list[int(i)].index for i in self.opt_id_list], dim=0
+        )  # shape: (K, num_lams)
+
         _, center_id = fft2(torch.zeros(self.mat_shape, dtype=dtype))
         self.M_arr = (
             self.m_arr.reshape(-1, 1) - self.m_arr.reshape(1, -1) + center_id[0]
@@ -305,6 +321,61 @@ class ETM:
                 order_id[i] = index[0, 0]  # 选择第一个满足条件的索引
         self.order_id = order_id
 
+    def convert_surf(self, x):
+        """
+        自动检测输入格式:
+        1D: surf_material_id (相对于 Material_list) → 返回 surf_logits (L,K)
+        2D: surf_logits (L,K) → 返回 surf_material_id (相对于 Material_list)
+
+        其中 K = number of True in is_opt_Material
+        """
+        x = torch.tensor(x, device=self.device)
+
+        opt_ids = self.opt_id_list  # 真实 Material_list 索引列表 (K,)
+        K = self.num_opt_mat
+
+        # ------------ case 1: 输入 1D，材料 ID → logits ------------ #
+        if x.dim() == 1:
+            L = x.numel()
+            logits = torch.zeros((L, K), device=self.device)
+
+            # 将 Material_list 编号的材料映射到优化材料维度
+            # 对于每层：找到 x[i] 在 opt_ids 中的位置
+            for i in range(L):
+                idx = (opt_ids == int(x[i])).nonzero(as_tuple=False)
+                if idx.numel() > 0:
+                    logits[i, idx[0, 0]] = 1.0  # one-hot
+                else:
+                    raise ValueError(f"材料 {int(x[i])} 不是可优化材料")
+
+            return logits
+
+        # ------------ case 2: 输入 2D logits → 材料 ID ------------ #
+        elif x.dim() == 2:
+            # softmax 最大 index → 对应 opt_ids 中的材料
+            max_idx = torch.argmax(x, dim=-1)  # (L,)
+            material_id = opt_ids[max_idx]  # 映射回 Material_list 编号
+            return material_id
+
+        else:
+            raise ValueError("convert_surf 输入必须为 1D 材料 ID 或 2D logits")
+
+    def compute_er_surf(self, beta):
+        # (L, K)
+        w = torch.softmax(self.surf_logits, dim=-1)
+        # 二值逼近
+        w_sig = sigmoid(w, beta)
+        w_norm = w_sig / (w_sig.sum(dim=-1, keepdim=True) + 1e-9)
+
+        # 使用预计算 allowed_n (K, num_lams)
+        allowed_n = self.allowed_n
+
+        # 加权插值
+        self.n_surf = (w_norm.unsqueeze(-1) * allowed_n.unsqueeze(0)).sum(dim=1)
+
+        # epsilon
+        self.er_surf = self.n_surf**2
+
     def Conv_mat(self, er_mat):
         # 输入的是(波长,x,y)
         er_fft, _ = fft2(er_mat)
@@ -362,11 +433,12 @@ class ETM:
         F = cat((cat((W, W), dim=-1), cat((-V, V), dim=-1)), dim=-2)
         return F, X
 
-    def Update(self):
+    def Update(self, beta=1000):
         surf_index = 0
         # 先从后界面往前界面走
+        self.compute_er_surf(beta)
         for z in range(self.stack_material_id.numel()):
-            if self.is_surf[-z - 1]:  # 如果是最后一面
+            if self.is_opt_surf[-z - 1]:  # 如果是最后一面
                 er_mat = (self.er_surf[-surf_index - 1] - self.er_stack[-z - 1]).view(
                     -1, 1, 1
                 ) * self.surf[-surf_index - 1].unsqueeze(0) + self.er_stack[
@@ -470,3 +542,6 @@ class ETM:
             t_out = self.t[..., order_id, :]
             r_out = self.r[..., order_id, :]
         return t_out, r_out
+
+    def get_discrete_surf_material_id(self):
+        return self.convert_surf(self.surf_logits)
