@@ -6,8 +6,8 @@ import torchopticsy.Utils as Utils
 import torchopticsy.Properties as Properties
 from torchopticsy.FDTD import Recorder, Source, Structure
 import torch.nn.functional as F
-import oticsyCUDA.FDTD as FDTD_CUDA_Dispersion
-import oticsyCUDA.FDTD4 as FDTD_CUDA_MONO
+import opticsyCUDA.FDTD as FDTD_CUDA_Dispersion
+import opticsyCUDA.FDTD4 as FDTD_CUDA_MONO
 
 dnds = 0.04
 # 非均匀插值函数有问题!!!!主要是周期性边界条件自动扩充了一格,实际上是不需要的因为统一扩充了,这是方便在非均匀网格和均匀网格之间变化
@@ -404,30 +404,26 @@ class FDTD_Solver(Properties.Geometry):
 
         for mat in self.Material_list:
             mat.Initialization(self)
-        if self.dispersion:
-            er_type = torch.complex64
-        else:
-            er_type = torch.float32
         self.er = torch.zeros(
             [len(self.Material_list), self.lams.numel()],
-            dtype=er_type,
+            dtype=torch.complex64,
             device=self.device,
         )
-        self.f_co = torch.zeros_like(self.er)
+        self.f_co = torch.zeros_like(self.er, dtype=torch.float32)
         self.er_mean = torch.zeros(
-            [len(self.Material_list)], dtype=er_type, device=self.device
+            [len(self.Material_list)], dtype=torch.complex64, device=self.device
         )
 
         # 原先的无补偿版本
-        for i in range(len(self.Material_list)):
-            self.er[i] = self.Material_list[i].er
-            self.er_mean[i] = torch.mean(self.Material_list[i].er)
+        # for i in range(len(self.Material_list)):
+        #     self.er[i] = self.Material_list[i].er
+        #     self.er_mean[i] = torch.mean(self.Material_list[i].er)
         # end
         # 暂定的数值色散版本
         lams = self.lams.view(-1)
         for i in range(len(self.Material_list)):
             self.er[i] = self.Material_list[i].er
-            n = torch.sqrt(self.er[i])
+            n = torch.abs(torch.sqrt(self.er[i]))
             f = torch.sinc(n * ds_2 * 2 / lams) / torch.sinc(Utils.c * self.dt / lams)
             self.f_co[i] = f
             self.er[i] *= f**2
@@ -797,14 +793,56 @@ class FDTD_Solver(Properties.Geometry):
                 structure.Initialization(self)
 
             FDTD_CUDA_MONO.Set_ER_average(
-                self.ERx, self.pri_weightx, self.pri_mat_idx, self.sec_mat_idx, self.er
+                self.ERx,
+                self.pri_weightx,
+                self.pri_mat_idx,
+                self.sec_mat_idx,
+                torch.real(self.er),
             )
             FDTD_CUDA_MONO.Set_ER_average(
-                self.ERy, self.pri_weighty, self.pri_mat_idy, self.sec_mat_idy, self.er
+                self.ERy,
+                self.pri_weighty,
+                self.pri_mat_idy,
+                self.sec_mat_idy,
+                torch.real(self.er),
             )
             FDTD_CUDA_MONO.Set_ER_average(
-                self.ERz, self.pri_weightz, self.pri_mat_idz, self.sec_mat_idz, self.er
+                self.ERz,
+                self.pri_weightz,
+                self.pri_mat_idz,
+                self.sec_mat_idz,
+                torch.real(self.er),
             )
+            k_list = (
+                2
+                * torch.pi
+                * Utils.c
+                / self.lams.view(-1)[0].cpu().item()
+                * self.dt
+                / 2
+            ) * torch.imag(self.er[:, 0])
+            FDTD_CUDA_MONO.Set_sigma_average(
+                self.sigmadt_2x,
+                self.pri_weightx,
+                self.pri_mat_idx,
+                self.sec_mat_idx,
+                k_list,
+            )
+            FDTD_CUDA_MONO.Set_sigma_average(
+                self.sigmadt_2y,
+                self.pri_weighty,
+                self.pri_mat_idy,
+                self.sec_mat_idy,
+                k_list,
+            )
+            FDTD_CUDA_MONO.Set_sigma_average(
+                self.sigmadt_2z,
+                self.pri_weightz,
+                self.pri_mat_idz,
+                self.sec_mat_idz,
+                k_list,
+            )
+
             # self.pri_mat_idx,self.pri_mat_idy,self.pri_mat_idz=E_Property_Periodic(self.pri_mat_idx,self.pri_mat_idy,self.pri_mat_idz,self.periodic_num)
             # self.sec_mat_idx,self.sec_mat_idy,self.sec_mat_idz=E_Property_Periodic(self.sec_mat_idx,self.sec_mat_idy,self.sec_mat_idz,self.periodic_num)
             self.ERx, self.ERy, self.ERz = E_Property_Periodic(
