@@ -1,6 +1,7 @@
 import torch
 import torch.nn.functional as F
 from torchopticsy.Properties import Geometry
+from torchopticsy.Utils import c
 import opticsyCUDA.FDTD as FDTD_CUDA_Dispersion
 import opticsyCUDA.FDTD4 as FDTD_MONO_CUDA
 
@@ -438,7 +439,7 @@ class Structure_MONO(Geometry):
         y_range = slice(int(y_int_id[0]), int(y_int_id[1]) + 1)
         z_range = slice(int(z_int_id[0]), int(z_int_id[1]) + 1)
         fdtd.sigmadt_2x[:, x_range, y_range, z_range] += (
-            self.sigma * weight * fdtd.dt / 2
+            (self.sigma * weight) * fdtd.dt / 2
         )
 
         weight, x_int_id, y_int_id, z_int_id = FDTD_MONO_CUDA.Mat_average(
@@ -454,7 +455,7 @@ class Structure_MONO(Geometry):
         y_range = slice(int(y_int_id[0]), int(y_int_id[1]) + 1)
         z_range = slice(int(z_int_id[0]), int(z_int_id[1]) + 1)
         fdtd.sigmadt_2y[:, x_range, y_range, z_range] += (
-            self.sigma * weight * fdtd.dt / 2
+            (self.sigma * weight) * fdtd.dt / 2
         )
 
         weight, x_int_id, y_int_id, z_int_id = FDTD_MONO_CUDA.Mat_average(
@@ -470,7 +471,7 @@ class Structure_MONO(Geometry):
         y_range = slice(int(y_int_id[0]), int(y_int_id[1]) + 1)
         z_range = slice(int(z_int_id[0]), int(z_int_id[1]) + 1)
         fdtd.sigmadt_2z[:, x_range, y_range, z_range] += (
-            self.sigma * weight * fdtd.dt / 2
+            (self.sigma * weight) * fdtd.dt / 2
         )
 
     def Set_subcell(self, fdtd):
@@ -487,7 +488,7 @@ class Structure_MONO(Geometry):
             fdtd.pri_mat_idx,
             fdtd.sec_mat_idx,
             fdtd.Nx,
-            fdtd.er,
+            torch.real(fdtd.er),
             0,
         )
 
@@ -504,7 +505,7 @@ class Structure_MONO(Geometry):
             fdtd.pri_mat_idy,
             fdtd.sec_mat_idy,
             fdtd.Ny,
-            fdtd.er,
+            torch.real(fdtd.er),
             1,
         )
 
@@ -521,7 +522,7 @@ class Structure_MONO(Geometry):
             fdtd.pri_mat_idz,
             fdtd.sec_mat_idz,
             fdtd.Nz,
-            fdtd.er,
+            torch.real(fdtd.er),
             2,
         )
         return
